function yi = interp1_self(x,y,xi)
%INTERP1 1-D interpolation (table lookup)
%   YI = INTERP1(X,Y,XI) interpolates to find YI, the values of the
%   underlying function Y at the points in the array XI. X must be a
%   vector of length N.
%   If Y is a vector, then it must also have length N, and YI is the
%   same size as XI.  If Y is an array of size [N,D1,D2,...,Dk], then
%   the interpolation is performed for each D1-by-D2-by-...-Dk value
%   in Y(i,:,:,...,:).
%   If XI is a vector of length M, then YI has size [M,D1,D2,...,Dk].
%   If XI is an array of size [M1,M2,...,Mj], then YI is of size
%   [M1,M2,...,Mj,D1,D2,...,Dk].
%
%   YI = INTERP1(Y,XI) assumes X = 1:N, where N is LENGTH(Y)
%   for vector Y or SIZE(Y,1) for array Y.
%
%   Interpolation is the same operation as "table lookup".  Described in
%   "table lookup" terms, the "table" is [X,Y] and INTERP1 "looks-up"
%   the elements of XI in X, and, based upon their location, returns
%   values YI interpolated within the elements of Y.
%
%   YI = INTERP1(X,Y,XI,METHOD) specifies alternate methods.
%   The default is linear interpolation. Use an empty matrix [] to specify
%   the default. Available methods are:
%
%     'nearest'  - nearest neighbor interpolation
%     'linear'   - linear interpolation
%     'spline'   - piecewise cubic spline interpolation (SPLINE)
%     'pchip'    - shape-preserving piecewise cubic interpolation
%     'cubic'    - same as 'pchip'
%     'v5cubic'  - the cubic interpolation from MATLAB 5, which does not
%                  extrapolate and uses 'spline' if X is not equally
%                  spaced.
%
%   YI = INTERP1(X,Y,XI,METHOD,'extrap') uses the interpolation algorithm
%   specified by METHOD to perform extrapolation for elements of XI outside the
%   interval spanned by X. YI = INTERP1(X,Y,XI,METHOD,EXTRAPVAL) replaces the
%   values outside of the interval spanned by X with EXTRAPVAL.  NaN and 0 are
%   often used for EXTRAPVAL.  The default extrapolation behavior with four
%   input arguments is 'extrap' for 'spline' and 'pchip' and EXTRAPVAL = NaN for
%   the other methods.
%
%   PP = INTERP1(X,Y,METHOD,'pp') will use the interpolation algorithm specified
%   by METHOD to generate the ppform (piecewise polynomial form) of Y. The
%   method may be any of the above METHOD except for 'v5cubic'. PP may then be
%   evaluated via PPVAL. PPVAL(PP,XI) is the same as
%   INTERP1(X,Y,XI,METHOD,'extrap').
%
%   For example, generate a coarse sine curve and interpolate over a
%   finer abscissa:
%       x = 0:10; y = sin(x); xi = 0:.25:10;
%       yi = interp1(x,y,xi); plot(x,y,'o',xi,yi)
%
%   For a multi-dimensional example, we construct a table of functional
%   values:
%       x = [1:10]'; y = [ x.^2, x.^3, x.^4 ];
%       xi = [ 1.5, 1.75; 7.5, 7.75]; yi = interp1(x,y,xi);
%
%   creates 2-by-2 matrices of interpolated function values, one matrix for
%   each of the 3 functions. yi will be of size 2-by-2-by-3.
%
%   Class support for inputs X, Y, XI, EXTRAPVAL:
%      float: double, single
%
%   See also INTERP1Q, INTERPFT, SPLINE, PCHIP, INTERP2, INTERP3, INTERPN, PPVAL.

%   Copyright 1984-2010 The MathWorks, Inc.
%   $Revision: 5.41.4.14 $  $Date: 2010/03/31 18:24:27 $
%
% Determine input arguments.

% The following syntaxes do not have X as the first input:
% INTERP1(Y,XI)
% INTERP1(Y,XI,METHOD)
% INTERP1(Y,XI,METHOD,'extrap')
% INTERP1(Y,XI,METHOD,EXTRAPVAL)

siz_y = size(y);
% y may be an ND array, but collapse it down to a 2D yMat. If yMat is
% a vector, it is a column vector.
n = siz_y(1);
ds = siz_y(2:end);
prodDs = prod(ds);
yMat = reshape(y,[n prodDs]);
xCol = x(:);


% Process XI in INTERP1(Y,XI,...) and INTERP1(X,Y,XI,...)
% Avoid syntax PP = INTERP1(X,Y,METHOD,'pp')
siz_xi = size(xi);

% The size of the output YI
if isvector(y)
    % Y is a vector so size(YI) == size(XI)
    siz_yi = siz_xi;
else
    if isvector(xi)
        % Y is not a vector but XI is
        siz_yi = [length(xi) ds];
    else
        % Both Y and XI are non-vectors
        siz_yi = [siz_xi ds];
    end
end

% xi may be an ND array, but flatten it to a column vector xiCol
xi = xi(:);


% Start the algorithm
% We now have column vector xCol, column vector or 2D matrix yMat and
% column vector xiCol.



% spline is oriented opposite to interp1
yiMat = spline(xCol.',yMat.',xi.').';


% Reshape result, possibly to an ND array
yi = reshape(yiMat,siz_yi);



end % INTERP1
